#include <QVector>
#include <QtMath>
#include <QPainter>
#include "inc/gravity_object.h"

gravity_object::gravity_object(QObject* parent) : QObject(parent) {
    position = {0.0, 0.0};
    velocity = {0.0, 0.0};
    mass = 0.0;
    gravity_constant = 6.67408 / 3.0;
}

void gravity_object::initialize(QVector<qreal> initial_pos, QVector<qreal> initial_vel, qreal initial_mass) {
    position = initial_pos;
    velocity = initial_vel;
    mass = initial_mass;
}

void gravity_object::update_velocity_influenced_by(gravity_object* other) {
    QVector<qreal> acceleration;

    acceleration = calc_resulting_acceleration(other);

    velocity[0] += acceleration[0];
    velocity[1] += acceleration[1];
}

void gravity_object::update_position(void) {
    position[0] += velocity[0];
    position[1] += velocity[1];
}

void gravity_object::paint(QPainter* painter) {
    QRect viewport = painter->viewport();

    QPoint origin_offset = QPoint(viewport.x() + (int) (viewport.width() / 2),
                                  viewport.y() + (int) (viewport.height() / 2));

    QPoint coordinates = QPoint((int)position[0], (int)position[1]) + origin_offset;

    QPoint velocity_endpoint = QPoint((int) (velocity[0] * 10), (int) (velocity[1] * 10)) + coordinates;

    painter->drawEllipse(coordinates, 5, 5);
    painter->drawLine(coordinates, velocity_endpoint);
}

QVector<qreal> gravity_object::calc_resulting_acceleration(gravity_object* other) {
    QVector<qreal> pos_difference{0.0, 0.0};
    qreal distance_scalar = 0.0;
    qreal acceleration_scalar = 0.0;
    QVector<qreal> resulting_acceleration{0.0, 0.0};

    if (this != other) { // only do calculation if not compared to self
        pos_difference[0] = other->position[0] - position[0];
        pos_difference[1] = other->position[1] - position[1];

        distance_scalar = qSqrt(qPow(pos_difference[0], 2.0) + qPow(pos_difference[1], 2.0));

        acceleration_scalar = gravity_constant * other->mass / (qPow(distance_scalar, 3.0));

        resulting_acceleration[0] = acceleration_scalar * pos_difference[0];
        resulting_acceleration[1] = acceleration_scalar * pos_difference[1];
    }

    return resulting_acceleration;
}
